Package: GBASS
Title: Generalized Bayesian Adaptive Smoothing Splines
Version: 2.0.1
Authors@R: 
    person(given = "Kellin",
           family = "Rumsey",
           role = c("aut", "cre"),
           email = "knrumsey@lanl.gov")
Description: Bayesian nonlinear regression under a range of likelihood models
    using generalized Bayesian adaptive smoothing splines. Robust regression
    with Student's t likelihoods, quantile regression, and related latent-scale
    models are included as special cases.
License: BSD_3_clause + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Depends: R (>= 3.5.0)
Imports: Matrix, GIGrvg, BASS
Suggests: knitr, rmarkdown, lhs, testthat (>= 2.1.0)
NeedsCompilation: no
Packaged: 2026-04-22 17:02:42 UTC; knrumsey
Author: Kellin Rumsey [aut, cre]
Maintainer: Kellin Rumsey <knrumsey@lanl.gov>
Repository: CRAN
Date/Publication: 2026-04-24 20:10:16 UTC
Built: R 4.6.0; ; 2026-04-26 03:43:13 UTC; unix
