# Generated by roxygen2: do not edit by hand

S3method(plot,gbass)
S3method(predict,gbass)
S3method(print,gbass)
export(build_GBP)
export(build_GIG)
export(build_prior)
export(gbass)
export(gbass2bass)
export(gm2bm)
export(gsobol)
export(hbass)
export(nw_est_mom)
export(nw_gamma_prior)
export(nw_triangle)
export(nwbass)
export(nwbass2)
export(qbass)
export(rgig2)
export(rmhn)
export(tbass)
import(Matrix)
importFrom(grDevices,adjustcolor)
importFrom(graphics,curve)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(stats,dnorm)
importFrom(stats,optim)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,rbinom)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,combn)
