#' @importFrom grDevices adjustcolor
#' @importFrom graphics curve hist legend par points segments
#' @importFrom stats dnorm optim pnorm predict rbinom rgamma rnorm runif sd var
#' @importFrom utils combn
"_PACKAGE"
